Package: rbmn
Version: 0.9-6
Date: 2023-06-28
Type: Package
Title: Handling Linear Gaussian Bayesian Networks
Author: Jean-Baptiste Denis [aut, cre], Marco Scutari [ctb]
Maintainer: Marco Scutari <scutari@bnlearn.com>
Description: Creation, manipulation, simulation of linear Gaussian Bayesian
             networks from text files and more...
License: GPL (>= 2)
Imports: MASS
Suggests: bnlearn, igraph
NeedsCompilation: no
Packaged: 2023-06-30 08:24:49 UTC; fizban
Repository: CRAN
Date/Publication: 2023-06-30 08:50:02 UTC
Built: R 4.6.0; ; 2026-04-25 21:35:22 UTC; unix
