export(var2pre,
     string7dag4nbn,
     state4chain,
     simulate8nbn,
     simulate8mn,
     simulate8gmn,
     simulate8gema,
     rmatrix4nbn,
     rm8nd4nbn,
     rm8nd4adja,
     reverse8chain,
     print8nbn,
     print8mn,
     print8gema,
     print8chain,
     order4nbn,
     order4gema,
     order4chain,
     normalize8nbn,
     nbn4rmatrix,
     nbn4nbn,
     nbn2rr,
     nbn2nbn,
     nbn2mn,
     nbn2gema,
     nbn2chain,
     nbn2bnfit,
     nb8bn,
     mn4joint1condi,
     mn2gema,
     marginal4chain,
     is8nbn8chain,
     inout4chain,
     generate8nbn,
     generate8chain,
     gema2nbn,
     gema2mn,
     estimate8nbn,
     estimate8constrainednbn,
     diff8nbn,
     dev4mn,
     crossed4nbn1nbn,
     cor4var,
     condi4joint,
     check8nbn,
     check8gema,
     check8chain,
     chain4chain,
     chain2pre,
     chain2nbn,
     chain2mn,
     chain2gema,
     chain2correlation,
     bnfit2nbn,
     bn2nbn,
     arcs4nbn1nbn,
     arc7nb4nbn,
     adja4three,
     adja4nbn,
     adja2nbn,
     adja2crossed,
     adja2arcs);
export(
       rbmn0chain.01,
       rbmn0chain.02,
       rbmn0chain.03,
       rbmn0nbn.01,
       rbmn0nbn.02,
       rbmn0nbn.03,
       rbmn0nbn.04,
       rbmn0nbn.05,
       rbmn0adja.01,
       rbmn0adja.02,
       rbmn0adja.03,
       rbmn0adja.04,
       rbmn0adja.05,
       rbmn0mn.01,
       rbmn0mn.02,
       rbmn0mn.03,
       rbmn0mn.04,
       rbmn0mn.05,
       rbmn0gema.01,
       rbmn0gema.02,
       rbmn0gema.03,
       rbmn0gema.04,
       rbmn0gema.05,
       rbmn0crarc.05
      );

importFrom("stats", "anova", "as.formula", "coefficients", "lm", "rbinom",
  "rnorm", "runif")
importFrom("utils", "head", "tail")
importFrom("MASS", "mvrnorm")
